use rusqlite::{named_params, Connection, Row};
use serde::Serialize;

use super::{
    error::DataStoreError,
    pagination::{Pagination, SortOrder},
};

#[derive(Debug, Serialize)]
pub struct Author {
    pub id: u64,
    pub name: String,
    pub sort: String,
}

impl Author {
    fn from_row(row: &Row<'_>) -> Result<Self, rusqlite::Error> {
        Ok(Self {
            id: row.get(0)?,
            name: row.get(1)?,
            sort: row.get(2)?,
        })
    }

    pub fn multiple(
        conn: &Connection,
        limit: u64,
        cursor: Option<&str>,
        sort_order: SortOrder,
    ) -> Result<Vec<Self>, DataStoreError> {
        let pagination = Pagination::new("sort", cursor, limit, sort_order);
        pagination.paginate(
            conn,
            "SELECT id, name, sort FROM authors",
            &[],
            Self::from_row,
        )
    }

    pub fn book_author(conn: &Connection, id: u64) -> Result<Self, DataStoreError> {
        let mut stmt = conn.prepare(
            "SELECT authors.id, authors.name, authors.sort FROM authors \
                INNER JOIN books_authors_link ON authors.id = books_authors_link.author \
                WHERE books_authors_link.book = (:id)",
        )?;
        let params = named_params! { ":id": id };
        Ok(stmt.query_row(params, Self::from_row)?)
    }
}
